<?php declare (strict_types=1);

namespace Infomaker\EverywareAdsParameters;

use Everyware\ProjectPlugin\Components\ComponentSettingsRepository;
use Everyware\ProjectPlugin\Components\SettingsProviders\CollectionDbProvider;
use Infomaker\Everyware\Support\Storage\CollectionDB;


use Infomaker\Everyware\Support\Environment;
// use Infomaker\Everyware\Support\Str;

/**
 * Handles all ajax request for Inserting, deleting and importing settings parameters
 *
 */
class PluginAdsParameters extends ComponentSettingsRepository
{
    private static $fields = [
        'ads_parameters' => [],
    ];
    public const OPTION_NAME = 'everyware_ads_parameters';

    /**
     * Instantiates parent constructor with CollectionDbProvider
     * and CollectionDB passing Option_name and parameters field
     */
    public function __construct()
    {
        parent::__construct(
            new CollectionDbProvider(
                new CollectionDB(static::OPTION_NAME),
                static::$fields
            )
        );
    }
    /**
     * Enqueues the admin.js script and add ajax actions for
     * deleting, adding and Importing parameters.
     *
     * @return void
     */
    public function init(): void
    {
        add_action('admin_enqueue_scripts', static function () {
            if (is_admin()) {
                $assetsPath = plugin_dir_url(__DIR__) . 'dist';
                $scriptVersion = Environment::isDev() ? false : GIT_COMMIT;
                wp_enqueue_script(
                    'everyware-ads-parameters',
                    "{$assetsPath}/js/admin.js",
                    ['jquery'],
                    $scriptVersion,
                    true
                );
                wp_localize_script(
                    'everyware-ads-parameters',
                    'ads_parameters_ajax',
                    [
                        'ajax_url' => admin_url(
                            'admin-ajax.php'
                        ),
                    ]
                );
            }
        });
        add_action('wp_ajax_delete_parameter_ads', [$this, 'ajaxDeleteParameter_ads']);
        add_action('wp_ajax_add_parameters_ads', [$this, 'ajaxAddParameters_ads']);
        add_action('wp_ajax_import_json_ads', [$this, 'ajaxJsonImport_ads']);
    }

    /**
     * Ajax for merging jsonData into current settings parameters.
     *
     * @return void
     */
    public function ajaxJsonImport_ads(): void
    {
        if (isset($_POST['data'])) {
            $importedParametersJson = stripslashes(html_entity_decode($_POST['data']));

            $importedParameters = json_decode($importedParametersJson, true);

            wp_send_json($this->ads_mergeParameters($importedParameters));
        }
    }

    /**
     * Ajax for deleteing a parameter. Requires id to be present in data variable.
     *
     * @return void
     */
    public function ajaxDeleteParameter_ads()
    {
        if (isset($_POST['data'])) {
            $savedParameters = $this->ads_getSavedParameters();
            $newParameters = $this->ads_deleteParameter($savedParameters, $_POST['data']);

            wp_send_json($this->ads_updateParameters($newParameters));
        }
    }

    /**
     * Ajax for adding parameters
     *
     * @return void
     */
    public function ajaxAddParameters_ads()
    {
        if (isset($_POST['form'])) {
            parse_str($_POST['form'], $searcharray);
           
            // print_r( $settings);
            wp_send_json($this->ads_updateParameters($searcharray['fields']));
        }
    }

    /**
     * Retrives all saved parameters
     *
     * @return array
     */
    private function ads_getSavedParameters(): array
    {
        $savedParameters = $this->getValue('ads_parameters');
        $keys = array_keys($savedParameters);
        $values = array_values($savedParameters);

        $combinedParameters['key'] = $keys;
        $combinedParameters['value'] = $values;

        return $combinedParameters;
    }

    /**
     * Merges saved and imported parameters.
     *
     * @param array $importedParameters The imported settings parameters
     *
     * @return bool
     */
    private function ads_mergeParameters(array $importedParameters)
    {
        $savedParameters = $this->ads_getSavedParameters();

        foreach ($importedParameters as $parameter) {
            array_push($savedParameters['key'], $parameter['key']);
            array_push($savedParameters['value'], $parameter['value']);
        }

        return $this->ads_updateParameters($savedParameters);
    }

    /**
     * Deletes a item from the collection by getting its index from the passed key
     *
     * @param string $parameters The key to the parameter that should be deleted
     * @param array  $key        The full collection of parameters
     *
     * @return array
     */
    private function ads_deleteParameter(array $parameters, string $key): array
    {
        $id = array_search($key, array_values((array)$parameters['key']));
        unset($parameters['key'][$id]);
        unset($parameters['value'][$id]);

        return $parameters;
    }

    /**
     * Iterates over all parameters and removes parameters with duplicate keys.
     *
     * @param array $parameters The array which will be checked for duplicate keys.
     *
     * @return array
     */
    private function ads_deleteDuplicates(array $parameters): array
    {
        $duplicates = [];
        foreach (array_count_values($parameters['key']) as $value => $parameterKey) {
            if ($parameterKey > 1) {
                $duplicates[] = $value;
            }
        }
        foreach ($duplicates as $duplicate) {
            $parameters = $this->ads_deleteParameter($parameters, $duplicate);
        }

        return $parameters;
    }

    /**
     * Updates the parameters settingsField.
     * Removes duplicate parameters before saving.
     *
     * @param array $submittedParameters Contains all parameters that should be saved
     *
     * @return bool
     */
    private function ads_updateParameters(array $submittedParameters): bool
    {
        $parameters = $this->ads_deleteDuplicates($submittedParameters);

        $validParameters = array_combine($parameters['key'], $parameters['value']);

        $settings = $this->get();

        $settings['ads_parameters'] = $validParameters;

        return $this->save($settings);
    }
}
